All of lore.kernel.org
 help / color / mirror / Atom feed
From: Luiz Capitulino <lcapitulino@redhat.com>
To: qemu-devel@nongnu.org
Cc: aliguori@us.ibm.com, armbru@redhat.com
Subject: [Qemu-devel] [PATCH 03/18] QMP: Don't use do_info()
Date: Thu, 16 Sep 2010 17:20:49 -0300	[thread overview]
Message-ID: <1284668464-15981-4-git-send-email-lcapitulino@redhat.com> (raw)
In-Reply-To: <1284668464-15981-1-git-send-email-lcapitulino@redhat.com>

Since its inception, QMP has been using HMP's do_info() function
to run query commands.

This was a bad choice, as it made do_info() more complex and
contributed to couple QMP and HMP.

This commit fixes that by doing the following changes:

  1. Introduce qmp_find_query_cmd() and use it to directly lookup
     the info_cmds table

  2. Introduce qmp_call_query_cmd() so that QMP code is able
     to call query handlers without using do_info()

  3. Drop do_info() usage

We need all the three changes in one shot so that we don't break
the calling of query commands in QMP.

Signed-off-by: Luiz Capitulino <lcapitulino@redhat.com>
---
 monitor.c |   39 +++++++++++++++++++++++++++++++--------
 1 files changed, 31 insertions(+), 8 deletions(-)

diff --git a/monitor.c b/monitor.c
index b2d8a2d..0ddc826 100644
--- a/monitor.c
+++ b/monitor.c
@@ -3387,6 +3387,11 @@ static const mon_cmd_t *monitor_find_command(const char *cmdname)
     return find_dispatch_table(mon_cmds, cmdname);
 }
 
+static const mon_cmd_t *qmp_find_query_cmd(const char *info_item)
+{
+    return find_dispatch_table(info_cmds, info_item);
+}
+
 static const mon_cmd_t *monitor_parse_command(Monitor *mon,
                                               const char *cmdline,
                                               QDict *qdict)
@@ -4326,6 +4331,24 @@ static QDict *qmp_check_input_obj(QObject *input_obj)
     return input_dict;
 }
 
+static void qmp_call_query_cmd(Monitor *mon, const mon_cmd_t *cmd)
+{
+    QObject *ret_data = NULL;
+
+    if (monitor_handler_is_async(cmd)) {
+        qmp_async_info_handler(mon, cmd);
+        if (monitor_has_error(mon)) {
+            monitor_protocol_emitter(mon, NULL);
+        }
+    } else {
+        cmd->mhandler.info_new(mon, &ret_data);
+        if (ret_data) {
+            monitor_protocol_emitter(mon, ret_data);
+            qobject_decref(ret_data);
+        }
+    }
+}
+
 static void handle_qmp_command(JSONMessageParser *parser, QList *tokens)
 {
     int err;
@@ -4333,8 +4356,9 @@ static void handle_qmp_command(JSONMessageParser *parser, QList *tokens)
     QDict *input, *args;
     const mon_cmd_t *cmd;
     Monitor *mon = cur_mon;
-    const char *cmd_name, *info_item;
+    const char *cmd_name, *query_cmd;
 
+    query_cmd = NULL;
     args = input = NULL;
 
     obj = json_parser_parse(tokens, NULL);
@@ -4360,16 +4384,13 @@ static void handle_qmp_command(JSONMessageParser *parser, QList *tokens)
     }
 
     /*
      * XXX: We need this special case until we get info handlers
      * converted into 'query-' commands
      */
     if (compare_cmd(cmd_name, "info")) {
         qerror_report(QERR_COMMAND_NOT_FOUND, cmd_name);
         goto err_out;
-    } else if (strstart(cmd_name, "query-", &info_item)) {
-        cmd = monitor_find_command("info");
-        qdict_put_obj(input, "arguments",
-                      qobject_from_jsonf("{ 'item': %s }", info_item));
+    } else if (strstart(cmd_name, "query-", &query_cmd)) {
+        cmd = qmp_find_query_cmd(query_cmd);
     } else {
         cmd = monitor_find_command(cmd_name);
     }
@@ -4392,7 +4413,9 @@ static void handle_qmp_command(JSONMessageParser *parser, QList *tokens)
         goto err_out;
     }
 
-    if (monitor_handler_is_async(cmd)) {
+    if (query_cmd) {
+        qmp_call_query_cmd(mon, cmd);
+    } else if (monitor_handler_is_async(cmd)) {
         err = qmp_async_cmd_handler(mon, cmd, args);
         if (err) {
             /* emit the error response */
@@ -4360,16 +4384,13 @@ static void handle_qmp_command(JSONMessageParser *parser, QList *tokens)
 
     /*
-     * XXX: We need this special case until we get info handlers
-     * converted into 'query-' commands
+     * XXX: We need this special case until QMP has its own dispatch table
      */
     if (compare_cmd(cmd_name, "info")) {
-- 
1.7.3.rc2.dirty

  parent reply	other threads:[~2010-09-16 20:21 UTC|newest]

Thread overview: 30+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2010-09-16 20:20 [Qemu-devel] [PATCH 00/18] Monitor: split HMP and QMP dispatch tables Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 01/18] Monitor: Introduce find_dispatch_table() Luiz Capitulino
2010-09-29  9:29   ` Markus Armbruster
2010-09-16 20:20 ` [Qemu-devel] [PATCH 02/18] QMP: handle_qmp_command(): Move 'cmd' sanity check Luiz Capitulino
2010-09-16 20:20 ` Luiz Capitulino [this message]
2010-09-29 11:13   ` [Qemu-devel] [PATCH 03/18] QMP: Don't use do_info() Markus Armbruster
2010-09-16 20:20 ` [Qemu-devel] [PATCH 04/18] Monitor: Drop QMP bits from do_info() Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 05/18] Monitor: Drop is_async_return() Luiz Capitulino
2010-09-29 11:21   ` Markus Armbruster
2010-09-29 13:28     ` Luiz Capitulino
2010-09-29 15:11       ` Anthony Liguori
2010-09-29 15:11       ` Anthony Liguori
2010-09-16 20:20 ` [Qemu-devel] [PATCH 06/18] Monitor: Convert do_info() back to HMP Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 07/18] Monitor: Introduce the qemu-monitor-qmp.hx file Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 08/18] QMP: Introduce qmp_find_cmd() Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 09/18] QMP: Introduce command dispatch table Luiz Capitulino
2010-09-29 11:31   ` Markus Armbruster
2010-09-29 13:47     ` Luiz Capitulino
2010-09-29 14:23       ` Markus Armbruster
2010-09-16 20:20 ` [Qemu-devel] [PATCH 10/18] QMP: Introduce query commands " Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 11/18] QMP: Simplify do_info_commands() Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 12/18] QMP: Small cleanup in handle_qmp_command() Luiz Capitulino
2010-09-16 20:20 ` [Qemu-devel] [PATCH 13/18] Monitor: Drop QMP info from the qemu-monitor.hx file Luiz Capitulino
2010-09-16 20:21 ` [Qemu-devel] [PATCH 14/18] Monitor: Drop monitor_cmd_user_only() Luiz Capitulino
2010-09-16 20:21 ` [Qemu-devel] [PATCH 15/18] Monitor: Rename monitor_handler_ported() Luiz Capitulino
2010-09-16 20:21 ` [Qemu-devel] [PATCH 16/18] Monitor: Rename monitor_handler_is_async() Luiz Capitulino
2010-09-16 20:21 ` [Qemu-devel] [PATCH 17/18] Monitor: Directly call QObject handlers Luiz Capitulino
2010-09-16 20:21 ` [Qemu-devel] [PATCH 18/18] QMP: Introduce qmp_call_cmd() Luiz Capitulino
2010-09-21 12:56 ` [Qemu-devel] [PATCH 00/18] Monitor: split HMP and QMP dispatch tables Anthony Liguori
2010-09-29 11:41   ` Markus Armbruster

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1284668464-15981-4-git-send-email-lcapitulino@redhat.com \
    --to=lcapitulino@redhat.com \
    --cc=aliguori@us.ibm.com \
    --cc=armbru@redhat.com \
    --cc=qemu-devel@nongnu.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.